# M4 macro for ICE (http://www.zeroc.com)
# Author:   Sven Matner 07 Jan 2004
# Modified: Walter Christian <wolti@sil.at>
# 
# Changelog:
# * Tue Sep 21, Walter Christian <wolti@sil.at>
#   - Changed the help strings for the AC_ARG_WITH to use AC_HELP_STRING
#   - Fixed warnings about underquoted strings which make problems with
#     recent version of autoconf
#   - Made a more modular system so that not all checks have to be done. 
#   - Let the user customize which variables are definied by suppliend a
#     variable prefix.
# * Fri Sep 24, Walter Christian <wolti@sil.at>
#   - Added more configuration options to specify includes, libraries and
#     exec prefix. 
#   - Searching for Ice in /opt 

# Description: Tests for configure arguments and makes sure that all prerequisites are
#   fullfilled.
# Usage: AC_PROG_ICE()
AC_DEFUN([AC_PROG_ICE],[
  AC_REQUIRE([AC_CANONICAL_HOST])
  AC_REQUIRE([AC_CANONICAL_BUILD])
  AC_ARG_WITH(
    ice,
    AC_HELP_STRING([--with-ice=PREFIX], [directory where ICE is installed]),
      [if test x${withval} != x"no" -a x${withval} != x"yes"; then
        ac_ice_prefix="${withval}"
      fi]
  )
  # First look if an default prefix is given. If no default prefix is given 

  # search for Ice in /opt/Ice-* or assume /usr/local
  if test x${ac_ice_prefix} = "x"; then
    ac_ice_prefix=/usr
    if test -d /opt; then
      AC_MSG_CHECKING([for Ice in /opt])
      ac_ice_possible_dirs=`ls /opt/ |egrep -e "Ice-.*"`
      for ac_ice_possible_dir in $ac_ice_possible_dirs; do
        if test -d "/opt/${ac_ice_possible_dir}" -a -x "/opt/${ac_ice_possible_dir}/bin/slice2cpp"; then
          ac_ice_prefix="/opt/${ac_ice_possible_dir}"
          break
        fi	
      done
      if test ${ac_ice_prefix} = "/usr"; then
        AC_MSG_RESULT([no])
      else
        AC_MSG_RESULT([yes])
      fi
    fi

    if test -d /usr/local; then
      AC_MSG_CHECKING([for Ice in /usr/local])
      if test -x /usr/local/bin/slice2cpp -a -d /usr/local/include/Ice; then
        ac_ice_prefix="/usr/local"
      fi
    fi
    if test ${ac_ice_prefix} != "/usr/local"; then
      if test -e ${ac_ice_prefix}/lib/libIce.so; then
	ac_ice_libdir=lib
      elif test -e ${ac_ice_prefix}/lib64/libIce.so; then
	ac_ice_libdir=lib64
      fi
      
      AC_MSG_RESULT([no])
    else
      AC_MSG_RESULT([yes])
    fi
  fi

  AC_ARG_WITH(
    ice-bindir,
    AC_HELP_STRING([--with-ice-bindir=DIR], [directory where ICE tools are installed]),
    [ac_ice_bindir="${withval}"],
    [ac_ice_bindir="${ac_ice_prefix}/bin"]
  )
  AC_ARG_WITH(
    ice-includes,
    AC_HELP_STRING([--with-ice-includes=DIR], [directory where the ICE headers are installed]),
    [ac_ice_includes="${withval}"],
    [ac_ice_includes="${ac_ice_prefix}/include"]
  )
  AC_ARG_WITH(
    ice-libs,
    AC_HELP_STRING([--with-ice-libs=DIR], [directory where the ICE headers are installed]),
    [ac_ice_libs="${withval}"],
    [ac_ice_libs="${ac_ice_prefix}/${ac_ice_libdir}"]
  )
  AC_MSG_NOTICE([The following ICE installation paths are assumed:
Ice prefix:   ${ac_ice_prefix}
Ice bindir:   ${ac_ice_bindir}
Ice includes: ${ac_ice_includes}
Ice libs:     ${ac_ice_libs}])
  AC_SUBST(LIB_PATH,[$ac_ice_libs])
  AC_SUBST(ICE_BIN_DIR,[$ac_ice_bindir])
])

# Description: Detect the ICE libraries and build an simple sample application
# to see if ice is working.
# Usage: AC_LIB_ICE(VARIABLEBASE, [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
# Result: Defines the variables VARIABLEBASE_CFLAGS, VARIABLEBASE_LIBS and
#   VARIABLEBASE_LDFLAGS
AC_DEFUN([AC_LIB_ICE],[
  AC_REQUIRE([AC_PROG_ICE])
  AC_REQUIRE([AC_LANG_CPLUSPLUS])
  AC_REQUIRE([AC_PROG_CXX])
  ac_succeeded=no

  # Store the old context of the CXX language so that we can modify the
  # variables.
  _cppflags="${CPPFLAGS}"
  _ldflags="${LDFLAGS}"

  $1_CFLAGS="-I$ac_ice_includes"
  $1_LIBS="-lIce -lIceUtil"
  $1_LDFLAGS="-L$ac_ice_libs"

  # On FreeBSD we need to link with the reentrant C library to prevent an
  # segmentation fault on startup.
  case $host_os in
    solaris*)
      $1_CFLAGS="${$1_CFLAGS} -D_REENTRANT"
      $1_LIBS="${$1_LIBS} -lpthread"
      ;;
    netbsd*)
      $1_CFLAGS="${$1_CFLAGS} -D_REENTRANT"
      $1_LIBS="${$1_LIBS} -lpthread"
      ;;
    freebsd*)
      if test -x /sbin/sysctl; then
        ac_freebsd_ver=`/sbin/sysctl -n kern.osreldate`
      else
        ac_freebsd_ver=`/usr/sbin/sysctl -n kern.osreldate`
      fi
      if test $ac_freebsd_ver -lt "500016"; then
        $1_CFLAGS="${$1_CFLAGS} -D_THREAD_SAFE"
	$1_LIBS="${$1_LIBS} -lpthread"
      else 
        if test $ac_freebsd_ver -lt "502102"; then
          $1_CFLAGS="${$1_CFLAGS} -D_THREAD_SAFE"
          $1_LIBS="${$1_LIBS} -lc_r"
        else 
          $1_LIBS="${$1_LIBS} -pthread"
        fi
      fi
      ;;
  esac
  
  CPPFLAGS="${CPPFLAGS} ${$1_CFLAGS}"
  LDFLAGS="${LDFLAGS} ${$1_LDFLAGS} ${$1_LIBS}"

  AC_CHECK_HEADER(
    Ice/Ice.h,
    [ac_succeeded=yes],
    [],
    [#undef INT64_MAX
  ])

  if test $ac_succeeded != yes; then
    ifelse([$3], , AC_MSG_ERROR([The ICE include files do no exist or could not be included. Try --with-ice]), [$3])
  fi

  AC_MSG_CHECKING([if ICE library is working])
  AC_LINK_IFELSE(
    [AC_LANG_PROGRAM(
      [[#include <Ice/Ice.h>]],
      [[int argc = 0;
char *argv[] = {"test"};
Ice::CommunicatorPtr ic;
try {
	ic = Ice::initialize(argc, argv);
} catch (const Ice::Exception & e) {
} catch (const char *msg) {
} 
if (ic) ic->destroy();]]
    )],
    [ac_succeeded=yes],
    [ac_succeeded=no]
  )

  if test $ac_succeeded = yes; then
    AC_MSG_RESULT([yes])
    ifelse([$2], , :, [$2])
  else
    AC_MSG_RESULT([no])
    ifelse([$3], , AC_MSG_ERROR([could not compile sample program. See config.log for more information.]), [$3])
  fi

  AC_SUBST($1_CFLAGS)
  AC_SUBST($1_LIBS)
  AC_SUBST($1_LDFLAGS)
  CPPFLAGS=${_cppflags}
  LDFLAGS=${_ldflags}
])

# Description: ICE software projects need the slice2cpp programm to generate C++ header
# and stub implementations out of the IDL file.
# Usage: AC_PROG_SLICE(VARIABLEBASE, [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
# Result: Defines the variable VARIABLEBASE_SLICE2CPP which points to the slice2cpp
#   executable.

AC_DEFUN([AC_PROG_SLICE],[
	AC_REQUIRE([AC_PROG_ICE])
	ac_succeeded=no
	ac_ice_paths="$ac_ice_bindir:${PATH}"
	AC_PATH_PROG(ac_slice2cpp, slice2cpp, no, $ac_ice_paths)
	if test -x "$ac_slice2cpp"; then
		ac_succeeded=yes
		ac_slice2cpp="tools/wrapper.sh ${ac_slice2cpp}"
	fi
	if test $ac_succeeded = yes; then
    	ifelse([$2], , :, [$2])
  	else
    	ifelse([$3], , AC_MSG_ERROR([The slice2cpp binary was not found in the search path '${ac_ice_paths}'.]), [$3])
  	fi
	AC_SUBST($1_SLICE2CPP, [${ac_slice2cpp}])
])

AC_DEFUN([AC_PROG_SLICE2PY],[
	AC_REQUIRE([AC_PROG_ICE])
	ac_succeeded=no
	ac_ice_paths="$ac_ice_bindir:${PATH}"
	AC_PATH_PROG(ac_slice2py, slice2py, no, $ac_ice_paths)
	if test -x "$ac_slice2py"; then
		ac_succeeded=yes
		ac_slice2py="tools/wrapper.sh ${ac_slice2py}"
	fi
	if test $ac_succeeded = yes; then
    	ifelse([$2], , :, [$2])
  	else
    	ifelse([$3], , AC_MSG_ERROR([The slice2py binary was not found in the search path '${ac_ice_paths}'.]), [$3])
  	fi
	AC_SUBST($1_SLICE2PY, [${ac_slice2py}])
])

# Description: ICE software projects might need the slice2freeze program when they
# want to use the persistence service provided by ICE.
# Usage: AC_PROG_FREEZE(VARIABLEBASE, [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
# Result: Defines the variable VARIABLEBASE_SLICE2FREEZE which points to the 
#   slice2freeze executable.

AC_DEFUN([AC_PROG_FREEZE],[
	AC_REQUIRE([AC_PROG_ICE])
	ac_succeeded=no
	ac_ice_paths="$ac_ice_bindir:${PATH}"
	AC_PATH_PROG(ac_slice2freeze, slice2freeze, no, $ac_ice_paths)
	if test -x "$ac_slice2freeze"; then
		ac_succeeded=yes
		ac_slice2freeze="tools/wrapper.sh ${ac_slice2freeze}"
	fi
	if test $ac_succeeded = yes; then
    	ifelse([$2], , :, [$2])
  	else
    	ifelse([$3], , AC_MSG_ERROR([The slice2freeze binary was not found in the search path '${ac_ice_paths}'.]), [$3])
  	fi
	AC_SUBST($1_SLICE2FREEZE, [${ac_slice2freeze}])
])
