/**
 * $Id: ehrprovider.h 155 2010-01-23 03:07:29Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief Main class
 */

#ifndef EHRPROVIDER_H
#define EHRPROVIDER_H

/* forward declarations */
class Configuration;
class DataManager;

#include <Ice/Ice.h>
#include "provideri.h"
#include "interprovideri.h"

/**
 * Main Class
 *
 * Registers all servants for ICE.
 *
 * The default ICE error handling is used (the program terminates on
 * interrupt)
 */
class EhrProvider
: virtual public Ice::Application
{
  public:
    /**
    * default ctor
    */
    EhrProvider()
      : Application(), m_conf(NULL), m_data(NULL), m_provider(NULL)
    {}

    /**
     * default dtor
     */
    ~EhrProvider();

    /**
     * Main server routine
     */
    virtual int run(int, char*[]);

    /**
     * Ctor-like setup for members
     * You have to call this before you use the members.
     * This function gives more flexibility in handling than a ctor.
     */
    int setup(int argc, char *argv[]);

    /**
     * get pointer to Configuration
     * @return pointer to configuration
     */
    Configuration *getConfiguration()
    {
      return m_conf;
    }

  private:
    /**
    * Inverse function to setup()
    *
    * Deletes all members. Called before setup and in dtor
    */
    void cleanup();

  private:
    Configuration     *m_conf;
    DataManager       *m_data;
    ProviderIPtr       m_provider;
    InterProviderIPtr  m_interprovider;
};

#endif

/* vim: set et sw=2 ts=2: */
