/**
 * $Id: interprovideri.h 146 2010-01-21 01:02:32Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief Implementation of Provider to Provider Interface
 */

#ifndef INTERPROVIDERI_H
#define INTERPROVIDERI_H

#include "datamanager.h"
#include <interprovider.h>

class InterProviderI
: virtual public Ehr::InterProvider
{
  public:
    /**
     * Ctor (empty)
     */
    InterProviderI(DataManager *data)
      : m_data(data)
    {}

    /**
     * Dtor (empty)
     */
    virtual ~InterProviderI()
    {}

    /**
     * handler for interprovider createPrescription-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual void createPrescription(const ::Ehr::CreatePrescriptionRequestPtr&,
        const ::Ehr::Signature&, const ::Ehr::Signature&, const Ice::Current&)
      throw(Ehr::EhrException);

    /**
     * handler for interprovider consumePrescription-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual void consumePrescription(const ::Ehr::ConsumePrescriptionRequestPtr&,
        const ::Ehr::Signature&, const ::Ehr::Signature&, const Ice::Current&)
      throw(Ehr::EhrException);

    /**
     * handler for interprovider listDocuments-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual ::Ehr::DocumentList listDocuments(const ::Ehr::ListDocumentsRequestPtr&,
        const ::Ehr::Signature&, const ::Ehr::Signature&, ::Ehr::UserType,
        const Ice::Current&)
      throw(Ehr::EhrException);

    /**
     * handler for interprovider findDocuments-request
     * see ice files for parameters
     * @throw Ehr::EhrException on error
     * @throw internal_error on internal error
     */
    virtual ::Ehr::DocumentList findDocuments(const ::Ehr::FindDocumentsRequestPtr&,
        const ::Ehr::Signature&, const ::Ehr::Signature&, ::Ehr::UserType,
        const Ice::Current&)
      throw(Ehr::EhrException);

  private:
    /**
     * Verifies provider, owner and timestamp of a request
     * @param req the request
     * @param providerSignature signature of provider who sent the req
     * @param ownerSignature signature of owner of documents
     * @throw Ehr::InvalidAccountException on unknown owner
     * @throw Ehr::SignatureException on invalid provider or owner signature
     * @throw Ehr::InvalidRequestException in invalid timestamp
     * @throw Ehr::EhrException on internal error
     */
    void checkRequest(::Ehr::ThirdPartyRequestPtr req,
        const Ehr::Signature& providerSignature,
        const Ehr::Signature& ownerSignature)
      throw(Ehr::EhrException);

  private:
    DataManager *m_data;
};

typedef IceUtil::Handle<InterProviderI> InterProviderIPtr;

#endif


/* vim: set et sw=2 ts=2: */
