/**
 * $Id: logger.h 146 2010-01-21 01:02:32Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel / Gruppe 1
 * @brief logger definitions
 */

#ifndef LOGGER_H
#define LOGGER_H

#include <log4cxx/logger.h>
#ifndef LOG4CXX_TRACE
/* this is log4cxx 0.9.7 */
# define __LOG4CXX_VERSION 907
# define NEED_LOG4CXX_TRACE

# undef PACKAGE
# undef PACKAGE_NAME
# undef PACKAGE_STRING
# undef PACKAGE_TARNAME
# undef PACKAGE_VERSION
# undef VERSION
#else
/* this is log4cxx 0.10.0 */
# define __LOG4CXX_VERSION 1000
#endif

/*----------------------------------------------------------------------------*/

#ifdef NEED_LOG4CXX_TRACE

#include <log4cxx/helpers/stringhelper.h>

namespace log4cxx
{
  class CustomLevel
  {
    DECLARE_LOG4CXX_LEVEL(CustomLevel);

    enum
    {
      TRACE_INT = 5000
    };

    static const LevelPtr TRACE;

    static LevelPtr getTrace();

    static const LevelPtr& toLevel(int val);

    static const LevelPtr& toLevel(int val, const LevelPtr& defaultLevel);

    static const LevelPtr& toLevel(const String& sArg);

    static const LevelPtr& toLevel(const String& sArg, const LevelPtr& defaultLevel);
  };
}

/**
Logs a message to a specified logger with the TRACE#CustomLevel level.

@param logger the logger to be used.
@param message the message string to log.
*/
#define LOG4CXX_TRACE(logger, message) { \
  if (logger->isEnabledFor(::log4cxx::CustomLevel::TRACE)) { \
    ::log4cxx::StringBuffer oss; \
      oss << message; \
        logger->forcedLog(::log4cxx::CustomLevel::TRACE, oss.str(), __FILE__, __LINE__); }}
#endif

/*----------------------------------------------------------------------------*/

#include <Ice/Logger.h>

/**
 * implements a logger wrapper around log4cxx for Ice
 */
class IceLogger
 : public Ice::Logger
{
  public:
    /**
     * loglevel: info
     * @param  message log message
     */
    void print(const std::string& message)
    {
      LOG4CXX_INFO(logger, message);
    }

    /**
     * loglevel: trace
     * @param  category log category
     * @param  message log message
     */
    void trace(const std::string& category, const std::string& message)
    {
      LOG4CXX_TRACE(logger, "" << category << ": " << message);
    }

    /**
     * loglevel: warning
     * @param  message log message
     */
    void warning(const std::string& message)
    {
      LOG4CXX_WARN(logger, message);
    }

    /**
     * loglevel: error
     * @param  message log message
     */
    void error(const std::string& message)
    {
      LOG4CXX_ERROR(logger, message);
    }

  private:
    /* should be static. Otherweise Ice will crash log4cxx upon destruction */
    static log4cxx::LoggerPtr logger;
};

#endif

/* vim: set et sw=2 ts=2: */
