/**
 * $Id: connectionpooltest.cpp 171 2010-01-30 17:49:36Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel
 * @brief Tests for connecion pool
 */

#define BOOST_TEST_MODULE connectionpooltest
#define BOOST_TEST_DYN_LINK
#include "redirector.h"
#include <boost/test/unit_test.hpp>
#include <boost/test/utils/runtime/env/variable.hpp>

#include <string>
#include <vector>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <IceUtil/Thread.h>
#include <boost/random.hpp>
#include "../configuration.h"
#include "../connectionpool.h"

#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)

using namespace std;
using namespace boost;
using namespace IceUtil;

typedef ConnectionPool CP;

struct Fixture
{
  Fixture()
  {
    /* create argv inside vector */
    vector<string>argvvec;
    argvvec.push_back(TOSTRING(BOOST_TEST_MODULE));
    argvvec.push_back("-f" TESTCONFDIR "/" TESTCONFFILE);
    optional<string> dbdsn;
    runtime::env::get("TEST_DBDSN", dbdsn);
    if (dbdsn)
      argvvec.push_back("-odatabase.dsn=\"" + dbdsn.get() + "\"");

    /* convert to c-style const char**. this is a bit hackish */
    unsigned argc = argvvec.size();
    char **argv = new char*[argc + 2];
    for(unsigned i = 0; i < argc; ++i)
      argv[i] = strdup(argvvec[i].c_str());
    argv[argc] = NULL;

    // conf setup
    m_conf = new Configuration(argc, const_cast<char **>(argv));
    BOOST_ASSERT(m_conf->loadCertificates());
  }

  Configuration *m_conf;
};

/*----------------------------------------------------------------------------*/

BOOST_FIXTURE_TEST_SUITE(connectionpool, Fixture)

BOOST_AUTO_TEST_CASE(nonthreaded)
{
  CP pool(m_conf->val<string>("database.dsn"), 1);
  /* fetch the same connection a few times. should be released automatically */
  for (unsigned i = 0; i < 3; ++i)
    CP::Connection conn = pool.get();
}

/*----------------------------------------------------------------------------*/

Mutex mutex;

BOOST_AUTO_TEST_CASE(threaded)
{
  class TestThread
    : public Thread
  {
    public:
      TestThread(CP &pool)
        : m_pool(pool)
      {}

      virtual void run()
      {
        /* wait for all threads to get started */
        Mutex::Lock sync(mutex);
        sync.release();
        for (unsigned i = 0; i < 3; ++i)
        {
          CP::Connection conn = m_pool.get();
          /* generate random number between 1 and 30 */
          unsigned random = (rand() % 30 + 1);
          getThreadControl().sleep(Time::milliSeconds(random * 100));
          std::cerr << ".";
          cerr.flush();
        }
      }

    private:
      CP &m_pool;
  };

  CP pool(m_conf->val<string>("database.dsn"), 5);
  srand(time(NULL));

  Mutex::Lock sync(mutex);
  vector<ThreadControl> threads;
  for (unsigned i = 0; i < 10; ++i)
  {
    ThreadPtr t = new TestThread(pool);
    threads.push_back(t->start());
  }

  sync.release();

  for (vector<ThreadControl>::iterator it = threads.begin();
      it != threads.end(); ++it)
    it->join();
  std::cerr << std::endl;
}

BOOST_AUTO_TEST_SUITE_END()

/* vim: set et sw=2 ts=2: */
