/**
 * $Id: redirector.h 78 2010-01-13 14:20:31Z l0728348 $
 *
 * Copyright 2009
 *
 * @author Christian, Bernhard, Manuel
 * @brief  redirects boost.test output to file
 */

#ifndef REDIRECTOR_H
#define REDIRECTOR_H

#define BOOST_TEST_NO_MAIN

#include <iostream>
#include <fstream>
#include <string>
#include <cassert>
#include <boost/test/unit_test.hpp>
#include <boost/test/results_reporter.hpp>
#include <boost/test/utils/runtime/env/variable.hpp>
#include "../logger.h"
#include <log4cxx/basicconfigurator.h>
#include <string.h>
#include <libgen.h>

/**
 * redirects boost.test output to file
 */
class ReportRedirector
{
  public:
    /**
     * default ctor
     */
    ReportRedirector(std::string file)
    {
      using namespace boost::unit_test;
      using namespace boost::runtime;

      boost::optional<std::string> enable;
      env::get("REDIRECTOR_ENABLE", enable);
      if (!enable)
        return;

      boost::optional<std::string> rfile;
      env::get("REDIRECTOR_FILE", rfile);
      if (!rfile && !file.empty())
      {
        char *filecpy1 = strdup(file.c_str());
        char *filecpy2 = strdup(file.c_str());
        std::string curdir  = dirname(filecpy2);
        std::string curfile = basename(filecpy1);
        free(filecpy1);
        free(filecpy2);

        rfile = curdir + "/results_" + curfile + ".xml";
      }

      if (rfile)
      {
        out.open(rfile->c_str());
        assert(out.is_open());
        results_reporter::set_stream(out);
      }
      else
        assert(!rfile);
    }

  private:
    std::ofstream out;
};


/*----------------------------------------------------------------------------*/

int main( int argc, char* argv[] )
{
  /* disable logging */
  log4cxx::BasicConfigurator::configure();
#if __LOG4CXX_VERSION >= 1000
  log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::getOff());
#else
  log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::OFF);
#endif

  static ReportRedirector redirector(argv[0]);
  return ::boost::unit_test::unit_test_main( &init_unit_test, argc, argv );
}

#endif

