#!/bin/sh

# OpenSSL CA script
# SE/Linux Team <se-linux $at$ inso \dot\ tu-wien \dot\ ac \dot\ at>

case "$1" in
  init)
    # Hard init, creates new key and CA certificate

    [ -d var ] || mkdir var

    [ -d work ] || mkdir work

    [ -d new_certs ] || mkdir new_certs

    sed -i -e "s,commonName = SE/Linux CA,commonName = CA of ${USER}," ca.cnf
    for file in ca.cnf req.cnf.template; do
      sed -i -e "s,0.organizationName = se-linux.inso.tuwien.ac.at,0.organizationName = SE-Linux student ${USER}," $file
    done

    rm -f var/*
    echo '01' > var/serial
    touch var/index
    openssl req -nodes -config ca.cnf -days 1825 -x509 -newkey rsa:2048 -out ca-cert.pem -outform PEM
    ;;
  _init)
    # Soft init, creates key and certificate from backup
    rm new_certs/* -f
    rm var/* -f
    echo '01' > var/serial
    touch var/index
    cp backup/ca-cert.pem .
    cp backup/ca-key.pem .
    ;;
  view)
    # Displays a certificate
    if [ -z "${2}" ]; then
      echo === Viewing CA root certificate === >&2
      openssl x509 -in ca-cert.pem -text
    else
      echo === Viewing certificate for CN ${2} === >&2
      openssl x509 -in work/${2}.pem -text
    fi
    ;;
  -h|--help|help)
    [ -f README.Students ] && cat README.Students >&2
    ;;
  *)
    if [ ! -d work ]; then
      echo "use \"$0 init\" first"
      exit 100
    fi
    cn="$1"
    echo Creating certificate with CN=$cn >&2
    sed "s/%CN%/$cn/" req.cnf.template > "work/$cn.cnf"
    openssl genrsa -out "work/$cn.key"
    openssl req -config "work/$cn.cnf" -new -nodes -key "work/$cn.key" -out "work/$cn.csr"
    openssl ca -batch -config ca.cnf -in "work/$cn.csr" -out "work/$cn.pem"
    cat "work/$cn.key" >> "work/$cn.pem"
    rm "work/$cn.cnf" "work/$cn.key" "work/$cn.csr"
    ;;
esac
